/*
 * $Id: config.cc,v 1.7 2005/08/22 02:12:20 amaia Exp $
 */

/**
 *	\file
 *	A classe config tem por objetivo reunir todos os parâmetros
 *  de execução (tanto os parâmetros da simulação como
 *  configurações de ambiente).
 *
 *	Inclui também facilidades que permitem que essas configurações
 *	sejam lidas tanto de um arquivo de configuração como através
 *	da linha de comando.
 *		
 */

#include <iostream>
#include <fstream>
#include <iterator>

#include <boost/program_options.hpp>
#include "version.h"
#include "modular.h"
#include "config.h"

namespace po = boost::program_options;
using std::string;
using std::vector;
using std::ostream;
using std::ostream_iterator;
using std::ifstream;
using std::endl;
using std::cout;
using std::cerr;


/**
 *
 * A helper function to simplify the main part.
 */
template<class T>
ostream& operator<<(ostream& os, const vector<T>& v)
{
    copy(v.begin(), v.end(), ostream_iterator<T>(cout, " ")); 
    return os;
}

/**
 * Ctor.
 *
 */
config::config( int argc, char** argv )
{
	init( argc, argv );
}

/**
 *	Ctor.
 */
config::config()
	:
	argc_(0),
	argv_(0)
{
}


void config::init( int argc, char** argv )
{
	argc_ = argc;
	argv_ = argv;
}

/**
 *
 *
 */
int config::read() 
{
		assert( argv_ != 0);
		
		//group of opts allowd on cmdline only
		po::options_description generic( "Generic options" );
		generic.add_options()
			("version,v", "print version string")
			("help", "produce help message")
			("config,c", 
				po::value<string>(&cfg_file_)->default_value("modular.cfg"),
				"parameters file")
			;

		//group of opts allowd both on cmdline and config file
		po::options_description cfg( "Simulation Parameters" );
		cfg.add_options()
			("N,N",
				po::value<u64>(&N_)->default_value(0),
				"valor N")
			("M,M",
				po::value<u64>(&M_)->default_value(0),
				"valor M")
			("tau",
				po::value<ldouble>(&tau_)->default_value(0),
				"tau")
			("U,U", 
				po::value<ldouble>(&U_)->default_value(0),
				"")
			("VEZ,V", 
				po::value<u64>(&VEZ_)->default_value(0),
				"")
			("S2", 
				po::value<ldouble>(&S2_)->default_value(0),
				"")
			("mtarget", 
				po::value<ldouble>(&mtarget_)->default_value(0),
				"")
			("Ttrans", 
				po::value<u64>(&Ttrans_)->default_value(0),
				"")
			("T,T", 
				po::value<u64>(&T_)->default_value(0),
				"")
			("Emax,", 
				po::value<ldouble>(&Emax_)->default_value(0),
				"")
			("sigma_gauss_q", 
				po::value<ldouble>(&sigma_gauss_q_)->default_value(0),
				"")
			("sigma_gauss_p",
				po::value<ldouble>(&sigma_gauss_p_)->default_value(0),
				"")
			;

		po::options_description cfg2( "Configuration Options" );
		cfg2.add_options()
			("output_dir,o",
				po::value<string>(&out_dir_)->default_value("out"),
				"diretorio de saida")
			("input_dir,i", 
				po::value<string>(&inp_dir_)->default_value("input"),
				"diretorio de entrada")
			("debug_level,d", 
				po::value<int>(&debug_level_)->default_value(0),
				"")
			;

		po::options_description cmdline_options;
		cmdline_options.add( generic )
					   .add( cfg )
					   .add( cfg2 );//.add( hidden );

		po::options_description cfgfile_options;
		cfgfile_options.add( cfg )
					   .add( cfg2 );

		po::options_description visible;
		visible.add( generic ).add( cfg )
							  .add(cfg2);

		po::positional_options_description p;
		p.add( "input_file", -1);

		po::variables_map vm;
        store( po::command_line_parser( argc_, argv_).
              options( cmdline_options ).positional( p ).run(), vm);

		notify( vm );
		
		if( vm.count("config") ) {
			ifstream ifs( cfg_file_.c_str() );
			if( !ifs ) {
				cerr << "cant open specified config file, aborting."
					 << endl;
				CkExit();
			}
			store( parse_config_file( ifs, cfgfile_options ), vm);
		}

		notify( vm );
   
        if( vm.count( "help" )) {
			cout << endl << endl;
			cout << "Modular:" << endl;
			cout << "Usage: <options> <input-files>" << endl;
            cout << visible << endl;
            CkExit();
        }

        if( vm.count( "version")) {
            cout << "version: " << current_version << endl;
            CkExit();
        }
}

/**
 * PUP
 */
void config::pup(PUP::er &p)
{
	p|N_;
	p|M_;
	p|tau_;
	p|U_;
	p|VEZ_;
	p|S2_;
	p|mtarget_;
	p|Ttrans_;
	p|T_;
	p|Emax_;
	p|sigma_gauss_p_;
	p|sigma_gauss_q_;
	p|cfg_file_;
	p|out_dir_;
	p|inp_dir_;
	p|debug_level_;

	//argc & argv doesnt need to be pupped
}
   
