/**
 * $Id: simulacao.cc,v 1.6 2005/08/22 03:07:18 amaia Exp $
 */

/**
 * \file 
 *
 * A classe simulacao representa uma realização. Possui parâmetros
 * e entrada próprios, controla um conjunto de objetos (rotor), pode
 * armazenar estados temporários e reune os resultados, contribuindo
 * para a classe Main ao fim da execução.
 *
 */

#include "modular.h"

using namespace std;

extern CProxy_Main mainProxy;
extern CkChareID mainhandle;
extern config gConfig;
extern int debug_verbosity;

/**
 * Inicializa os vetores p, q com N elementos a partir do
 * arquivo de entrada.
 *
 * @param filename		[IN] Nome do arquivo
 * @param p				[OUT] valores de p
 * @param q				[OUT] valores de q
 * @param N				[IN] quantidade de elementos a serem lidos do arquivo
 */
void initial_mom_coor(string filename, vdouble& p, vdouble& q, u64 N)
{
	timer t1;
	t1.start();
	
//	debug_print(DEBUG_INFO2,"init_mom_coor() - start\n"); 

	p.clear();
	q.clear();
	p.reserve(N);
	q.reserve(N);
	
	if (load_pq(filename, p, q) < 0) {
	
		debug_print(DEBUG_FATAL,
			"Error reading file (%s).\n"
			,filename.c_str());
		
		CkAbort("");
		
	}
	
	t1.stop();
//	debug_print(DEBUG_INFO2,"init_mom_coor() (t= %f secs)\n",t1.get());
}

/**
 * Construtor que dá suporte à funcionalidade de migração.
 * \todo 
 * implementar. escrever também as rotinas de pup.
 *
 */
simulacao::simulacao(CkMigrateMessage *m)
{ 

	/** empty */
}

/**
 * Construtor.
 *
 * Inicializa os atributos da simulação, utizando o objeto global de 
 * configuração.
 *
 */
simulacao::simulacao()
	:
	N_(gConfig.get_N()),
	M_(gConfig.get_M()),
	T_(gConfig.get_T()),
	Ttrans_(gConfig.get_Ttrans()),
	tau_(gConfig.get_tau()),
	G_(gConfig.get_N()), // por default, setar G como sendo N 
	t_(0)
{ 
	v_EK_	.reserve(T_);
	v_VM_	.reserve(T_);
	v_Vi_	.reserve(T_);
	v_m_	.reserve(T_);
	v_V_	.reserve(T_);
	v_K_	.reserve(T_);

	
	//a1=CProxy_A1::ckNew(parameters,nElements); )
	//@TODO: refactor
	const int nelem = (int)(N_/G_) + (N_ % G_ == 0 ? 0 : 1);
	
	debug_print(DEBUG_INFO2,
		"simulacao::simulacao() creating %d objects\n",
		nelem);
	
	rotorArray_ = CProxy_rotor::ckNew(thisIndex, G_, nelem);
	init_rotors(gConfig.get_inp_dir());

}

/**
 * Inicializa o array de rotores pertencente à simulação.
 *
 * @param input		[IN] string com o path do diretório onde estão os 
 *					arquivos de onde serão lidos os valores de p e q.
 */
void simulacao::init_rotors(string input)
{
	
//	debug_print(DEBUG_INFO2,"simul::init_rotors()\n");

	//@TODO: refactor
	const int nelem = (int)(N_/G_) + (N_ % G_ == 0 ? 0 : 1);

	//inicializar p e q
	vdouble p, q;
	vdouble_it pit, qit;

	ostringstream filename;
	filename << input << "/input." << thisIndex;

	debug_print(DEBUG_INFO2, 
		"simul::init_rotors(), opening %s\n",
		filename.str().c_str());

	initial_mom_coor(filename.str(), p, q, N_);
	assert(p.size() == q.size());
	
	pit = p.begin();
	qit = q.begin();
	int n;
	for (int i=0; i < nelem; i++) {

		// n sera ou _G ou a qtdade de elems q faltam
		//@TODO: refactor
		n = (i == nelem -1) ? N_ - i*G_ : G_;
		
		rotorArray_[i].set(vdouble(pit, pit+n), vdouble(qit,qit+n) );
		
		//debug_print(
		//	"simul::init_rotors() - setting rotor[%d] n=%d\n", i, n);
		
		pit += n;
		qit += n;
	}	
//	debug_print(DEBUG_INFO2,"simul::init_rotors()-end\n");
	return;
}

/**
 *	Inicia a simulação.
 *
 */
void simulacao::start()
{
	//debug_print(DEBUG_INFO2,"simul::start()\n");
	timer1_.start();
	rotorArray_.calc();
}

/**
 * Callback chamado pelo Rotor ao fim da simulação.
 * Ao término da simulação (todos os timesteps), esse métodos será
 * chamado pelo array de Rotor, passando os resultados obtidos.
 *
 * @param m 		[IN]
 */
void simulacao::cbDone1(CkReductionMsg* m)
{
	ldouble V, VM, Vi, K, KM;
	ldouble* data = (ldouble*) m->getData();
	assert( m->getSize() == 6*sizeof(ldouble));	
	
	v_V_	.push_back	(*data++);
	v_VM_	.push_back	(*data++);
	v_Vi_	.push_back	(*data++);
	v_K_	.push_back	(*data++);
	v_EK_	.push_back	(*data++);
	v_m_	.push_back	(*data++);
	
	delete m;

//	debug_print(DEBUG_INFO1,"simulacao::cbDone1()\n");
//	debug_print(DEBUG_EXTRA,
//		"\tV=%Lf,VM=%Lf,Vi=%Lf,K=%Lf,EK=%Lf,m=%Lf\n"
//		,v_V[_t],v_VM[_t],v_Vi[_t],v_K[_t],v_EK[_t],v_m[_t]);

	t_++;

	//último timestep
	if( t_ == T_) {
		
		timer1_.stop();
		
		debug_print(DEBUG_INFO2,
			"\tsimulacao %d done. (%f secs)\n", 
			thisIndex, timer1_.get());
			
		timer1_.start();
		
		mainProxy.cbSimulDone(v_EK_, v_VM_, v_Vi_, v_m_, v_V_, v_K_);
	}
}

/**
 *	Callback chamado ao fim da simulação.
 * 	Da mesma forma que cbDone1, porém são passados todos os estados dos
 *	rotores durante toda a simulação, para fins de depuração.
 *
 *	Este método não é mais necessário.. por isso está comentado.
 *
 */
void simulacao::cbDone2(CkReductionMsg* m)
{

#if 0
	//Esse metodo eh o Callback do Reduction que reune
	//todos os (p,q) dos rotores no obj simulacao, caso
	//haja necessidade de imprimir (debugging,etc).

	static int x =0;
	vdouble p(_N), q(_N);
	int idx; 

	debug_print(DEBUG_INFO2, "simulacao %d iteração t=%d\n",thisIndex,x);
	
	CkReduction::setElement* cur = (CkReduction::setElement*) m->getData();
	while(cur!=NULL) {

		int size = cur->dataSize/sizeof(ldouble);
		ldouble* data = (ldouble*)cur->data;

		// o primeiro elemento é o índice do
		// rotor que possui esses valores de p e q.
		idx		= *data++; size--;
		debug_print(DEBUG_INFO1,"simulacao::cbDone2() - idx=%d, size=%d \n",idx,size+1);
		assert(idx >= 0 && idx < _N);

		//base=real_index
		int base = idx*_G;
		//debug_print("idx=%d size=%d\n",base,size);
		for(int i=0; i<size; i+=2) {
			p[base] = *data++;
			q[base] = *data++;
			//debug_print("idx=%d p %f q %f\n",base,p[base],q[base]);
			base++;
		}
		
		cur=cur->next();
	}


	
	//nao eh mais necessario, usnado quiescence detection
	//if(x==1000) { mainProxy.done(); }

	ostringstream filename;
	filename << "out-" << thisIndex << "." << x;
	save_pq(filename.str(),p,q);
	x++;
	delete m;

#endif

}

/**
 *	Rotina PUP da classe simulação.
 *
 *	@param p		[IN]
 */
void simulacao::pup(PUP::er &p)
{
	//desatualizado!
	ArrayElement1D::pup(p);
	p|N_;
	p|M_;
	p|T_;
	p|Ttrans_;
	p|tau_;	
	p|rotorArray_;

}

